@@ -15,9 +15,11 @@ import py
 import pytest
 
 from .django_compat import is_django_unittest  # noqa
+from .fixtures import django_assert_num_queries  # noqa
 from .fixtures import django_db_setup  # noqa
 from .fixtures import django_db_use_migrations  # noqa
 from .fixtures import django_db_keepdb  # noqa
+from .fixtures import django_db_createdb  # noqa
 from .fixtures import django_db_modify_db_settings  # noqa
 from .fixtures import django_db_modify_db_settings_xdist_suffix  # noqa
 from .fixtures import _live_server_helper  # noqa
@@ -33,14 +35,15 @@ from .fixtures import settings  # noqa
 from .fixtures import transactional_db  # noqa
 from .pytest_compat import getfixturevalue
 
-from .lazy_django import (django_settings_is_configured,
-                          get_django_version, skip_if_no_django)
+from .lazy_django import django_settings_is_configured, skip_if_no_django
 
 
 SETTINGS_MODULE_ENV = 'DJANGO_SETTINGS_MODULE'
 CONFIGURATION_ENV = 'DJANGO_CONFIGURATION'
 INVALID_TEMPLATE_VARS_ENV = 'FAIL_INVALID_TEMPLATE_VARS'
 
+PY2 = sys.version_info[0] == 2
+
 
 # ############### pytest hooks ################
 
@@ -62,10 +65,10 @@ def pytest_addoption(parser):
                      help='Set DJANGO_CONFIGURATION.')
     group._addoption('--nomigrations', '--no-migrations',
                      action='store_true', dest='nomigrations', default=False,
-                     help='Disable Django 1.7+ migrations on test setup')
+                     help='Disable Django migrations on test setup')
     group._addoption('--migrations',
                      action='store_false', dest='nomigrations', default=False,
-                     help='Enable Django 1.7+ migrations on test setup')
+                     help='Enable Django migrations on test setup')
     parser.addini(CONFIGURATION_ENV,
                   'django-configurations class to use by pytest-django.')
     group._addoption('--liveserver', default=None,
@@ -258,7 +261,7 @@ def pytest_configure():
     _setup_django()
 
 
-def _method_is_defined_at_leaf(cls, method_name):
+def _classmethod_is_defined_at_leaf(cls, method_name):
     super_method = None
 
     for base_cls in cls.__bases__:
@@ -268,7 +271,15 @@ def _method_is_defined_at_leaf(cls, method_name):
     assert super_method is not None, (
         '%s could not be found in base class' % method_name)
 
-    return getattr(cls, method_name).__func__ is not super_method.__func__
+    method = getattr(cls, method_name)
+
+    try:
+        f = method.__func__
+    except AttributeError:
+        pytest.fail('%s.%s should be a classmethod' % (cls, method_name))
+    if PY2 and not (inspect.ismethod(method) and method.__self__ is cls):
+        pytest.fail('%s.%s should be a classmethod' % (cls, method_name))
+    return f is not super_method.__func__
 
 
 _disabled_classmethods = {}
@@ -280,9 +291,9 @@ def _disable_class_methods(cls):
 
     _disabled_classmethods[cls] = (
         cls.setUpClass,
-        _method_is_defined_at_leaf(cls, 'setUpClass'),
+        _classmethod_is_defined_at_leaf(cls, 'setUpClass'),
         cls.tearDownClass,
-        _method_is_defined_at_leaf(cls, 'tearDownClass'),
+        _classmethod_is_defined_at_leaf(cls, 'tearDownClass'),
     )
 
     cls.setUpClass = types.MethodType(lambda cls: None, cls)
@@ -440,7 +451,11 @@ def _django_set_urlconf(request):
     if marker:
         skip_if_no_django()
         import django.conf
-        from django.core.urlresolvers import clear_url_caches, set_urlconf
+        try:
+            from django.urls import clear_url_caches, set_urlconf
+        except ImportError:
+            # Removed in Django 2.0
+            from django.core.urlresolvers import clear_url_caches, set_urlconf
 
         validate_urls(marker)
         original_urlconf = django.conf.settings.ROOT_URLCONF
@@ -482,10 +497,25 @@ def _fail_for_invalid_template_variable(request):
             """There is a test for '%s' in TEMPLATE_STRING_IF_INVALID."""
             return key == '%s'
 
-        def _get_template(self):
+        @staticmethod
+        def _get_origin():
+            stack = inspect.stack()
+
+            # Try to use topmost `self.origin` first (Django 1.9+, and with
+            # TEMPLATE_DEBUG)..
+            for f in stack[2:]:
+                func = f[3]
+                if func == 'render':
+                    frame = f[0]
+                    try:
+                        origin = frame.f_locals['self'].origin
+                    except (AttributeError, KeyError):
+                        continue
+                    if origin is not None:
+                        return origin
+
             from django.template import Template
 
-            stack = inspect.stack()
             # finding the ``render`` needle in the stack
             frame = reduce(
                 lambda x, y: y[3] == 'render' and 'base.py' in y[1] and y or x,
@@ -501,18 +531,18 @@ def _fail_for_invalid_template_variable(request):
             # ``django.template.base.Template``
             template = f_locals['self']
             if isinstance(template, Template):
-                return template
+                return template.name
 
         def __mod__(self, var):
             """Handle TEMPLATE_STRING_IF_INVALID % var."""
-            template = self._get_template()
-            if template:
+            origin = self._get_origin()
+            if origin:
                 msg = "Undefined template variable '%s' in '%s'" % (
-                    var, template.name)
+                    var, origin)
             else:
                 msg = "Undefined template variable '%s'" % var
             if self.fail:
-                pytest.fail(msg, pytrace=False)
+                pytest.fail(msg)
             else:
                 return msg
 
@@ -520,7 +550,7 @@ def _fail_for_invalid_template_variable(request):
             django_settings_is_configured()):
         from django.conf import settings as dj_settings
 
-        if get_django_version() >= (1, 8) and dj_settings.TEMPLATES:
+        if dj_settings.TEMPLATES:
             dj_settings.TEMPLATES[0]['OPTIONS']['string_if_invalid'] = (
                 InvalidVarException())
         else:
@@ -536,7 +566,7 @@ def _template_string_if_invalid_marker(request):
         if marker and django_settings_is_configured():
             from django.conf import settings as dj_settings
 
-            if get_django_version() >= (1, 8) and dj_settings.TEMPLATES:
+            if dj_settings.TEMPLATES:
                 dj_settings.TEMPLATES[0]['OPTIONS']['string_if_invalid'].fail = False
             else:
                 dj_settings.TEMPLATE_STRING_IF_INVALID.fail = False
@@ -581,7 +611,7 @@ class _DatabaseBlocker(object):
 
     @property
     def _dj_db_wrapper(self):
-        from .compat import BaseDatabaseWrapper
+        from django.db.backends.base.base import BaseDatabaseWrapper
 
         # The first time the _dj_db_wrapper is accessed, we will save a
         # reference to the real implementation.
